/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;
import static com.google.gwt.query.client.plugins.Effects.Effects;

import org.dussan.vaadin.dtable.client.Translation;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.dom.client.Style.Display;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.user.client.Element;

public class LoadingMessage extends TableComponent {

	private String mainId = "div#v-dtable";
	private String bodyId = "table#dt-body-table";
	private String mainContainerWrapperId = "div#dt-main-container-wrapper";
	private String messageId = "div#dt-message";

	public LoadingMessage(VDTable dTable) {
		super(dTable);
	}

	@Override
	public void setIds() {
		if (!mainId.endsWith("" + getDTable().getId())) {
			mainId += getDTable().getId();
			bodyId += getDTable().getId();
			messageId += getDTable().getId();
		}
	}

	@Override
	public void load() {
		setIds();
		if (getDTable().config().isEnabledLoadingMessage()) {
			$(messageId)
					.empty()
					.append($("<div/>")
							.id("dt-message-wrapper")
							.addClass("wrapper")
							.css(CSS.DISPLAY.with(Display.NONE))
							.append($("<div/>")
									.text(getDTable()
											.translation()
											.get(Translation.LOADING_DATA_PLEASE_WAIT))
									.addClass("context")));
		} else {
			unload();
		}
		enableEventsHandler();
	}

	@Override
	public void unload() {
		$(messageId).remove();
		disableEventsHandler();
	}

	public void show() {
		if (getDTable().config().isEnabledLoadingMessage()) {
			GQuery wrapper = $(mainId).find(mainContainerWrapperId);
			$(messageId)
					.css("display", "")
					.children("div")
					.css("display", "")
					.parent()
					.css(CSS.LEFT.with(Length.px(0)))
					.css(CSS.TOP.with(Length.px(0)))
					.css(CSS.WIDTH.with(Length.px(wrapper.width())))
					.css(CSS.HEIGHT.with(Length.px(wrapper.height())))
					.css(CSS.ZINDEX.with(10000))
					.as(Effects)
					.animate("opacity:1",
							getDTable().config().getLoadingMessageShowDelay());
		}
	}

	public void sendMessage(final String eventType, final Object eventMessage) {
		if (getDTable().config().isEnabledLoadingMessage()) {
			$(bodyId)
					.parent()
					.parent()
					.parent()
					.as(Effects)
					.animate(
							"opacity:"
									+ getDTable().config()
											.getLoadingMessageOpacity(),
							getDTable().config().getLoadingMessageShowDelay(),
							new Function() {
								public void f() {
									getDTable().sentMessage(eventType,
											eventMessage);
								}
							});
		}
	}

	public void hide() {
		if (getDTable().config().isEnabledLoadingMessage()) {
			GQuery wrapper = $(mainId).find(mainContainerWrapperId);
			$(messageId).as(Effects).animate("opacity:0",
					getDTable().config().getLoadingMessageHideDelay(),
					new Function() {
						public void f(Element e) {
							$(e).css(CSS.DISPLAY.with(Display.NONE))
									.css(CSS.LEFT.with(Length.px(0)))
									.css(CSS.TOP.with(Length.px(0)))
									.css(CSS.WIDTH.with(Length.px(0)))
									.css(CSS.HEIGHT.with(Length.px(0)))
									.css(CSS.ZINDEX.with(-1));
						}
					});
			wrapper.children("table")
					.children("tr")
					.as(Effects)
					.animate("opacity:1",
							getDTable().config().getLoadingMessageHideDelay());
		}
		getDTable().enableEventsHandler();
	}

	@Override
	public void enableEventsHandler() {
	}

	@Override
	public void disableEventsHandler() {
	}

}
