/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.List;

import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.vaadin.terminal.gwt.client.UIDL;

public class Sorting extends TableComponent {

	private String mainId = "div#v-dtable";

	private List<String> sorting = new ArrayList<String>();

	public Sorting(VDTable dTable) {
		super(dTable);
	}

	private int getSortingColumn(String sorting) {
		return Integer.parseInt(sorting.split(" ")[0]);
	}

	private String getSortingType(String sorting) {
		return sorting.split(" ")[1];
	}

	private void setSortings(String[] sorting) {
		if (sorting != null && sorting.length > 0) {
			this.sorting.clear();
			if (!"none".equals(sorting[0])) {
				for (String sort : sorting) {
					sort = sort.toLowerCase();
					int column = getSortingColumn(sort);
					if (getDTable().isFrozen(column)
							&& getDTable().config()
									.isEnabledSortedFrozenTable()) {
						this.sorting.add(sort);
					} else if (!getDTable().isFrozen(column)
							&& getDTable().config().isEnabledSortedTable()) {
						this.sorting.add(sort);
					}
				}
			}
		}
	}

	public boolean isSortingsEnabled(int column) {
		return ((getDTable().getAllRows() > 1) && (!getDTable()
				.isFrozen(column) ? getDTable().config().isEnabledSortedTable()
				: getDTable().config().isEnabledSortedFrozenTable()));
	}

	public int getSize(boolean isTableFrozen) {
		if (sorting.isEmpty()) {
			return 0;
		} else {
			int columnsCounter = 0;
			for (int col = 0; col < sorting.size(); col++) {
				int column = getSortingColumn(sorting.get(col));
				if (isTableFrozen && getDTable().isFrozen(column)) {
					columnsCounter += 1;
				} else if (!isTableFrozen && !getDTable().isFrozen(column)) {
					columnsCounter += 1;
				}
			}
			return columnsCounter;
		}
	}

	public String getSortings() {
		StringBuilder builder = new StringBuilder();
		for (String sorting : this.sorting) {
			builder.append(sorting.toUpperCase()).append("_:_");
		}
		return (builder.length() == 0 ? "none" : builder.substring(0,
				builder.length() - 3));
	}

	public void setSortings(GQuery td, String sorting) {
		if (td != null && !td.isEmpty()) {
			int columnId = Integer.parseInt(td.id().split(":")[2]);
			if (sorting == null) {
				for (int col = this.sorting.size() - 1; col >= 0; col--) {
					int column = getSortingColumn(this.sorting.get(col));
					if (getDTable().isFrozen(columnId)
							&& getDTable().isFrozen(column)) {
						this.sorting.remove(col);
					} else if (!getDTable().isFrozen(columnId)
							&& !getDTable().isFrozen(column)) {
						this.sorting.remove(col);
					}
				}
			} else {
				sorting = sorting.toLowerCase();
				int column = -1;
				for (int col = 0; col < this.sorting.size(); col++) {
					if (getSortingColumn(this.sorting.get(col)) == columnId) {
						column = col;
					}
				}

				if ("asc".equals(sorting) || "desc".equals(sorting)) {
					if (column >= 0) {
						this.sorting.set(column, columnId + " " + sorting);
					} else {
						this.sorting.add(columnId + " " + sorting);
					}
				} else if ("remove".equals(sorting) && column >= 0) {
					this.sorting.remove(column);
				}
			}
		}
	}

	@Override
	public void setIds() {
		if (!mainId.endsWith("" + getDTable().getId())) {
			mainId += getDTable().getId();
		}
	}

	public void readUIDL(UIDL uidl) {
		if (uidl.hasAttribute("sorting")) {
			setSortings(uidl.getStringArrayAttribute("sorting"));
		} else {
			unload();
		}
	}

	@Override
	public void load() {
		setIds();
		enableEventsHandler();
	}

	@Override
	public void unload() {
		disableEventsHandler();
	}

	public void apply() {
		if ((getDTable().config().isEnabledSortedFrozenTable() || getDTable()
				.config().isEnabledSortedTable())
				&& this.sorting != null
				&& !this.sorting.isEmpty()) {
			String tdId = "td [id='" + getDTable().getId() + "h:0:_']";
			for (String sorting : this.sorting) {
				int column = getSortingColumn(sorting);
				if ((getDTable().config().isEnabledSortedFrozenTable() && getDTable()
						.isFrozen(column))
						|| (getDTable().config().isEnabledSortedTable() && !getDTable()
								.isFrozen(column))) {
					GQuery td = $(tdId.replace("_", "" + column));
					$(td).removeClass("asc", "desc")
							.children("div")
							.css(CSS.MARGIN_LEFT.with(Length.px(getDTable()
									.config().getHeaderCellMarginLeft())));
					$(td).addClass(getSortingType(sorting).toLowerCase())
							.children("div")
							.css(CSS.MARGIN_LEFT.with(Length.px(getDTable()
									.config().getHeaderCellMarginLeft()
									+ getDTable().config()
											.getHeaderCellMarginLeftSorting())));
				}
			}
		} else {
			unload();
		}
	}

	@Override
	public void enableEventsHandler() {
	}

	@Override
	public void disableEventsHandler() {
	}
}
