/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.vaadin.terminal.gwt.client.ui.VOverlay;

public class Tooltip extends VOverlay {

	private boolean enableWordWrapInTooltip = false;
	private int minWidth = 0;
	private int maxWidth = 0;
	private String text = new String();
	private String postition = "bottom";

	public Tooltip() {
		getElement().setInnerText("");
		getElement().setId("dt-tooltip");
		getElement().setClassName("dt-tooltip");
	}

	private void setText(String text) {
		$("<div />").html(text)
				.addClass(enableWordWrapInTooltip ? "normal" : "nowrap")
				.appendTo($(getElement()));
	}

	public void setTo(Event e, String text, boolean enableWordWrapInTooltip,
			int minWidth, int maxWidth, String postition) {
		this.text = text;
		this.enableWordWrapInTooltip = enableWordWrapInTooltip;
		this.minWidth = minWidth;
		this.maxWidth = maxWidth;
		this.postition = postition;

		Element element = DOM.eventGetTarget(e);
		if (element.getId().split(":").length > 3) {
			String[] id = element.getId().split(":");
			element = DOM.getElementById(id[0] + ":" + id[1] + ":" + id[2]);
		}

		if ("BOTTOM".equals(postition)) {
			setPopupPosition(DOM.eventGetClientX(e),
					DOM.getAbsoluteTop(element) + $(element).height() + 2);
		} else {
			setPopupPosition(DOM.eventGetClientX(e),
					DOM.getAbsoluteTop(element));
		}
	}

	@Override
	public void setPopupPosition(int left, int top) {
		if (getElement().getChildCount() == 0) {
			super.setPopupPosition(left, top);
		}
	}

	@Override
	public void show() {
		if (getElement().getChildCount() == 0 && !text.isEmpty()) {
			setText(text);
			super.show();

			// tooltip width
			minWidth = Math.min(minWidth, maxWidth);
			if (minWidth > 0) {
				$("#" + getElement().getId()).css(
						CSS.MIN_WIDTH.with(Length.px(minWidth)));
			}
			if (maxWidth > 0) {
				$("#" + getElement().getId()).css(
						CSS.MAX_WIDTH.with(Length.px(maxWidth)));
			}
			if ((Window.getClientWidth() - 30) < getOffsetWidth()) {
				$("#" + getElement().getId()).width(
						Window.getClientWidth() - 30);
			}

			// tooltip position
			int popupLeft = (getAbsoluteLeft() + getOffsetWidth() + 10) <= Window
					.getClientWidth() ? getPopupLeft() : (Window
					.getClientWidth() - getOffsetWidth() - 10);
			int popupTop = getPopupTop()
					- ("TOP".equals(postition) ? (getOffsetHeight() + 2) : 0);
			super.setPopupPosition(popupLeft, popupTop);
		}
	}

	@Override
	public void hide() {
		super.hide();
		if (getElement().getChildCount() > 0) {
			getElement().getChild(0).removeFromParent();
		}
	}

}
