/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.events;

import static com.google.gwt.query.client.GQuery.$;

import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.dom.client.Style.Display;
import com.google.gwt.dom.client.Style.Position;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.BorderStyleProperty.BorderStyle;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.query.client.css.RGBColor;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;

public class TableEvent {

	private static void removeExtendedCell(GQuery tr,
			boolean isEnabledHorizontalLines) {
		tr.children("td").each(new Function() {
			public void f(Element td) {
				$(td).children("div#clone").remove();
				$(td).children("div.normal", "div.nowrap").css(CSS.DISPLAY, "");
			}
		});
		if (!isEnabledHorizontalLines) {
			tr.css("border-top", "").css("border-bottom", "");
		}
		tr.removeClass("hover");
	}

	private static String getColId(String idString) {
		if (idString != null && idString.indexOf(":") >= 0
				&& idString.split(":").length >= 2) {
			String[] id = idString.split(":");
			return id[0] + ":" + id[1] + ":" + id[2];
		}
		return "";
	}

	private static String getRowId(String idString) {
		if (idString != null && idString.indexOf(":") >= 0
				&& idString.split(":").length > 1) {
			String[] id = idString.split(":");
			return id[0] + ":" + id[1];
		}
		return "";
	}

	private static boolean isHeader(String id) {
		return id.endsWith("h");
	}

	private static boolean isExpandedCellAlwaysVisible(VDTable dTable,
			String[] id) {
		int column = Integer.parseInt(id[2]);
		if (isHeader(id[0])) {
			if (dTable.isFrozen(column)) {
				return dTable.config()
						.isEnabledExpandedFrozenHeaderCellAlwaysVisible();
			}
			return dTable.config().isEnabledExpandedHeaderCellAlwaysVisible();
		} else {
			if (dTable.isFrozen(column)) {
				return dTable.config()
						.isEnabledExpandedFrozenBodyCellAlwaysVisible();
			}
			return dTable.config().isEnabledExpandedBodyCellAlwaysVisible();
		}
	}

	public static void mouseLeaveWindow(GQuery frozenHeader, GQuery header,
			GQuery frozenBody, GQuery body,
			final boolean isEnabledHorizontalLines) {
		frozenHeader.children("tr").each(new Function() {
			public void f(Element tr) {
				removeExtendedCell($(tr), isEnabledHorizontalLines);
			}
		});
		header.children("tr").each(new Function() {
			public void f(Element tr) {
				removeExtendedCell($(tr), isEnabledHorizontalLines);
			}
		});
		frozenBody.children("tr.hover").each(new Function() {
			public void f(Element tr) {
				removeExtendedCell($(tr), isEnabledHorizontalLines);
			}
		});
		body.children("tr.hover").each(new Function() {
			public void f(Element tr) {
				removeExtendedCell($(tr), isEnabledHorizontalLines);
			}
		});
	}

	public static boolean contextMenu(VDTable dTable, Event e) {
		if (!$(e).hasClass("search")
				&& !DOM.eventGetCurrentTarget(e).getInnerHTML()
						.contains("<img ")) {
			dTable.menu().add(DOM.eventGetCurrentTarget(e));
			dTable.menu().setPopupPosition(DOM.eventGetClientX(e),
					DOM.eventGetClientY(e));
			dTable.menu().show();
		}
		return false;
	}

	public static void cellMouseOver(VDTable dTable, GQuery td, String addClass) {
		final String[] id = td.id().split(":");
		if (id != null && id.length >= 3) {
			GQuery divBase = $(td).parent().parent().parent();
			GQuery table = divBase.children("table");
			GQuery tdLast = table.children("tr").last().children("td").last();
			GQuery div = $(td).children("div");
			GQuery span = div.children("span");

			int left = td.position().left + (!isHeader(id[0]) ? 1 : 0);
			int top = td.position().top + (!isHeader(id[0]) ? 1 : 0);
			int divWidth = div.outerWidth(true);
			int divHeight = div.outerHeight(true);

			int divInnerWidth = divBase.innerWidth();
			int divInnerHeight = divBase.innerHeight();
			int tableWidth = tdLast.position().left + divWidth;
			int tableHeight = tdLast.position().top + divHeight;
			int scrollLeft = table.position().left;
			int scrollTop = table.position().top;

			String margin = div.css(CSS.MARGIN_TOP);
			int marginTop = Integer.parseInt(margin.substring(0,
					margin.indexOf("px")));
			margin = div.css(CSS.MARGIN_RIGHT);
			int marginRight = Integer.parseInt(margin.substring(0,
					margin.indexOf("px")));
			margin = div.css(CSS.MARGIN_BOTTOM);
			int marginBottom = Integer.parseInt(margin.substring(0,
					margin.indexOf("px")));
			margin = div.css(CSS.MARGIN_LEFT);
			int marginLeft = Integer.parseInt(margin.substring(0,
					margin.indexOf("px")));

			int spanWidth = span.outerWidth() + marginLeft + marginRight;
			int spanHeight = span.outerHeight() + marginTop + marginBottom;

			boolean showExapndedCell = true;
			// check if expanded cell is smaller than table
			showExapndedCell &= spanWidth <= tableWidth;
			showExapndedCell &= spanHeight <= tableHeight;
			// check if expanded cell is larger than base cell
			showExapndedCell &= divWidth < spanWidth || divHeight < spanHeight;
			// check if base cell is completely visible
			if (!isExpandedCellAlwaysVisible(dTable, id)) {
				showExapndedCell &= (left + scrollLeft) >= 0;
				showExapndedCell &= (top + scrollTop) >= 0;
				showExapndedCell &= (left + divWidth + scrollLeft - 1 + dTable
						.getBrowserOffset()) <= divInnerWidth;
				showExapndedCell &= (top + divHeight + scrollTop - 1 + dTable
						.getBrowserOffset()) <= divInnerHeight;
			}

			if (showExapndedCell) {
				GQuery divOrig = div.clone();

				// check if expanded cell is over table right edge
				if (((!isExpandedCellAlwaysVisible(dTable, id) && (left
						+ spanWidth + scrollLeft) > divInnerWidth) || (left + spanWidth) > tableWidth)
						&& divWidth < spanWidth) {
					div.css(CSS.BORDER_RIGHT_STYLE.with(BorderStyle.NONE)).css(
							CSS.RIGHT.with(Length.px(0)));
				} else {
					div.css(CSS.BORDER_LEFT_STYLE.with(BorderStyle.NONE)).css(
							CSS.LEFT.with(Length.px(0)));
				}

				// check if expanded cell is over table bottom edge
				if (((!isExpandedCellAlwaysVisible(dTable, id) && (top
						+ spanHeight + scrollTop) > divInnerHeight) || (top + spanHeight) > tableHeight)
						&& divHeight < spanHeight) {
					div.css(CSS.BORDER_BOTTOM_STYLE.with(BorderStyle.NONE))
							.css(CSS.BOTTOM.with(Length.px(0)));
				} else {
					div.css(CSS.BORDER_TOP_STYLE.with(BorderStyle.NONE)).css(
							CSS.TOP.with(Length.px(0)));
				}

				if (divHeight < spanHeight) {
					div.css(CSS.HEIGHT.with(Length.px(spanHeight)));
					if (divWidth < spanWidth) {
						div.css(CSS.WIDTH.with(Length.px(spanWidth)));
					} else {
						div.css(CSS.WIDTH.with(Length.px(divWidth))).css(
								CSS.BORDER_RIGHT_STYLE.with(BorderStyle.NONE));
					}
				} else if (divWidth < spanWidth) {
					div.css(CSS.BORDER_BOTTOM_STYLE.with(BorderStyle.NONE));
					div.css(CSS.WIDTH.with(Length.px(spanWidth))).css(
							CSS.HEIGHT.with(Length.px(divHeight)));
				}

				span.css(CSS.WIDTH, "")
						.css(CSS.MARGIN.with(Length.px(marginTop),
								Length.px(marginRight),
								Length.px(marginBottom), Length.px(marginLeft)));

				int zindex = addClass == "focus" ? 1001 : 101;
				String clazz = addClass == "focus" ? addClass : "notfocus";
				div.addClass("expanded", "focus")
						.css(CSS.MARGIN.with(Length.px(0)))
						.css(CSS.POSITION.with(Position.ABSOLUTE))
						.wrap($("<div/>").id("clone").addClass(clazz)
								.css(CSS.ZINDEX.with(zindex))
								.css(CSS.POSITION.with(Position.RELATIVE))
								.css(CSS.WIDTH.with(Length.px(divWidth)))
								.css(CSS.HEIGHT.with(Length.px(divHeight))));

				divOrig.css(CSS.DISPLAY.with(Display.NONE)).appendTo($(td));
			}
		}
	}

	public static boolean headerCellMouseOut(Event e) {
		String idFrom = getColId(DOM.eventGetFromElement(e).getId());
		String idTo = getColId(DOM.eventGetToElement(e).getId());
		if (idFrom.equals(idTo)) {
			return false;
		}

		$(e).children("div#clone").not("div.focus").remove();
		$(e).children("div.normal", "div.nowrap").not("div.focus")
				.css(CSS.DISPLAY, "");
		return true;
	}

	public static void headerCellMouseOut(String tr, String id0, String id2,
			String removeClass) {
		String id = "td [id='" + id0 + "h:0:" + id2 + "']";
		if (removeClass != null && "focus".equals(removeClass)) {
			$(tr).children(id).children("div.focus").remove();
		}
		$(tr).children(id).children("div#clone").not("div.focus").remove();
		$(tr).children(id).children("div.normal", "div.nowrap")
				.not("div.focus").css(CSS.DISPLAY, "");
	}

	public static void headerClick(VDTable dTable, GQuery td) {
		if (!td.hasClass("search")) {
			int column = Integer.parseInt(td.id().split(":")[2]);
			if (((dTable.isFrozen(column) && dTable.config()
					.isEnabledSortedFrozenTable()))
					|| (!dTable.isFrozen(column) && dTable.config()
							.isEnabledSortedTable())) {
				dTable.sorting().setSortings(td,
						td.hasClass("asc") ? "desc" : "asc");
				dTable.sendMessage("sorting", dTable.sorting().getSortings(),
						true);
			}
		}
	}

	public static void bodyRowMouseOver(GQuery body, GQuery remainedBody,
			GQuery e, boolean isEnabledHorizontalLines) {
		if (!e.hasClass("empty") && body.children("tr").size() > 1) {
			int counter = Integer.parseInt(e.children("td").eq(0).id()
					.split(":")[1]);

			if (e.parent().children("tr").eq(0).hasClass("empty")) {
				counter += 1;
			}

			if (isEnabledHorizontalLines) {
				e.css(CSS.BORDER_TOP_COLOR.with(RGBColor.TRANSPARENT)).css(
						CSS.BORDER_BOTTOM_COLOR.with(RGBColor.TRANSPARENT));
				remainedBody
						.children("tr")
						.eq(counter)
						.css(CSS.BORDER_TOP_COLOR.with(RGBColor.TRANSPARENT))
						.css(CSS.BORDER_BOTTOM_COLOR.with(RGBColor.TRANSPARENT));
			}

			e.addClass("hover");
			remainedBody.children("tr").eq(counter).addClass("hover");
		}
	}

	public static boolean bodyRowMouseOut(GQuery body, GQuery remainedBody,
			Event event, boolean isEnabledHorizontalLines) {
		if (!$(event).hasClass("empty") && body.children("tr").size() > 1) {
			TableEvent.headerCellMouseOut(event);

			String idFrom = getRowId(DOM.eventGetFromElement(event).getId());
			String idTo = getRowId(DOM.eventGetToElement(event).getId());
			if (idFrom.equals(idTo)) {
				return false;
			}

			if (!isEnabledHorizontalLines) {
				$(event).css("border-top", "").css("border-bottom", "");
				remainedBody.children("tr.hover").css("border-top", "")
						.css("border-bottom", "");
			}

			$(event).removeClass("hover");
			remainedBody.children("tr.hover").removeClass("hover");
		}
		return true;
	}

	public static boolean bodyCellMouseOut(String headerTr, Event e) {
		TableEvent.headerCellMouseOut(e);

		String idFrom = getColId(DOM.eventGetFromElement(e).getId());
		String idTo = getColId(DOM.eventGetToElement(e).getId());
		if (idFrom.equals(idTo)) {
			return false;
		}

		String[] id = $(e).id().split(":");
		$(e).children("div#clone").remove();
		$(e).children("div.normal", "div.nowrap").css(CSS.DISPLAY, "");
		headerCellMouseOut(headerTr, id[0], id[2], null);
		return true;
	}

}
