/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.menu;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.vaadin.terminal.gwt.client.ui.VOverlay;

public class Menu extends VOverlay {

	private int column = 0;
	private int row = 0;
	private String cellValue = new String();
	private VDTable dTable = null;
	private Element menu = null;
	private Element targetE = null;

	public Menu(VDTable dTable) {
		this.dTable = dTable;

		getElement().setClassName("dt-context-menu");
		setAutoHideEnabled(true);
		setShadowEnabled(true);

		if (DOM.getChildCount(getElement()) > 0) {
			getElement().getFirstChildElement().removeFromParent();
		}

		sinkEvents(Event.ONCLICK | Event.ONCONTEXTMENU);
	}

	private VDTable getDTable() {
		return dTable;
	}

	public void add(Element e) {
		if (menu != null) {
			menu.removeFromParent();
		}

		targetE = e;
		menu = DOM.createDiv();
		getElement().appendChild(menu);

		String[] id = e.getId().split(":");
		cellValue = $(e).children("div").children("span").html().trim();
		Element item = null;
		List<MenuItems> menuItems = new ArrayList<MenuItems>();

		// table header
		if (id.length >= 3 && id[0].endsWith("h")) {
			column = Integer.parseInt(id[2]);

			// ascending/descending/remove sorting
			if (getDTable().sorting().isSortingsEnabled(column)) {
				String sortingType = MenuItems.SORTING_REMOVE.id();
				if ($(e).hasClass("asc")) {
					sortingType = MenuItems.SORTING_ASCENDING.id();
				} else if ($(e).hasClass("desc")) {
					sortingType = MenuItems.SORTING_DESCENDING.id();
				}

				menuItems
						.addAll(Arrays.asList(MenuItems.SORTING_ASCENDING,
								MenuItems.SORTING_DESCENDING,
								MenuItems.SORTING_REMOVE));
				for (MenuItems menuItem : menuItems) {
					String description = getDTable().translation().get(
							menuItem.description());
					if (!menuItem.id().equalsIgnoreCase(sortingType)) {
						item = DOM.createDiv();
						item.setId(menuItem.id());
						item.setInnerText(description.replace("_", cellValue));
						menu.appendChild(item);
					}
				}

				if (($(e).parent().children("td.asc", "td.desc").length() == 1
						&& !$(e).hasClass("asc") && !$(e).hasClass("desc"))
						|| getDTable().sorting().getSize(
								getDTable().isFrozen(column)) > 1) {
					item = DOM.createDiv();
					item.setId(MenuItems.SORTING_REMOVE_ALL.id());
					item.setInnerText(getDTable().translation().get(
							MenuItems.SORTING_REMOVE_ALL.description()));
					menu.appendChild(item);
				}
			}

			// freeze/unfreeze column
			if (getDTable().config().isEnabledFreezeUnfreezeColumns()
					&& getDTable().config().isEnabledFrozenTable()) {
				item = DOM.createDiv();
				item.setId(!getDTable().isFrozen(column) ? MenuItems.FREEZE
						.id() : MenuItems.UNFREEZE.id());
				item.setInnerText((!getDTable().isFrozen(column) ? dTable
						.translation().get(MenuItems.FREEZE.description())
						: getDTable().translation().get(
								MenuItems.UNFREEZE.description())).replace("_",
						cellValue));

				if (!getDTable().isFrozen(column)
						&& getDTable().config().isEnabledFreeze(column)
						&& getDTable().getColumnsVisible().size() > 1) {
					item.setInnerText(getDTable().translation()
							.get(MenuItems.FREEZE.description())
							.replace("_", cellValue));
					menu.appendChild(item);
				} else if (getDTable().isFrozen(column)
						&& getDTable().config().isEnabledUnfreeze(column)
						&& ((getDTable().config().isEnabledEmptyFrozenTable()
								&& !getDTable().isAnyFrozenColumnHidden() && dTable
								.getColumnsFrozen().size() >= 1) || (!dTable
								.config().isEnabledEmptyFrozenTable() && dTable
								.getColumnsFrozen().size() > 1))) {
					item.setInnerText(getDTable().translation()
							.get(MenuItems.UNFREEZE.description())
							.replace("_", cellValue));
					menu.appendChild(item);
				}
			}

			// hide column
			if (getDTable().config().isEnabledHideUnhideColumns()
					&& getDTable().config().isColumnHideEnabled(column)) {
				item = DOM.createDiv();
				item.setId(MenuItems.HIDE.id());
				item.setInnerText(getDTable().translation()
						.get(MenuItems.HIDE.description())
						.replace("_", cellValue));
				if (!getDTable().isFrozen(column)
						&& getDTable().getColumnsVisible().size() > 1) {
					menu.appendChild(item);
				} else if (getDTable().isFrozen(column)
						&& getDTable().getColumnsFrozen().size() > 1) {
					menu.appendChild(item);
				}
			}
		} else if (id.length >= 3) { // table body
			column = Integer.parseInt(id[2]);
			row = Integer.parseInt(id[1]);

			if (getDTable().filtering().isFilteringEnabled(column)) {
				if (cellValue != null
						&& cellValue.length() > 0
						&& !"&nbsp;".equals(cellValue)
						&& !getDTable().filtering().filterOperatorExists(
								column, MenuItems.IS_EMPTY.description())
						&& !getDTable().searchColumns().filterOperatorExists(
								column, MenuItems.IS_EMPTY.description())) {
					if (getDTable().config().isColumnFormatted(column)
							&& !getDTable().filtering().filterOperatorExists(
									column, MenuItems.EQUAL.description())
							&& !getDTable().searchColumns()
									.filterOperatorExists(column,
											MenuItems.IS_EMPTY.description())) {
						menuItems.addAll(Arrays.asList(MenuItems.EQUAL,
								MenuItems.NOT_EQUAL, MenuItems.GREATER,
								MenuItems.GREATER_EQUAL, MenuItems.LESS,
								MenuItems.LESS_EQUAL));
					} else if (!getDTable().config().isColumnFormatted(column)
							&& !getDTable().filtering().filterOperatorExists(
									column, MenuItems.LIKE.description())
							&& !getDTable().searchColumns()
									.filterOperatorExists(column,
											MenuItems.IS_EMPTY.description())) {
						menuItems.addAll(Arrays.asList(MenuItems.LIKE,
								MenuItems.LIKE_LEFT, MenuItems.LIKE_RIGHT,
								MenuItems.LIKE_LEFT_RIGHT, MenuItems.NOT_LIKE,
								MenuItems.NOT_LIKE_LEFT,
								MenuItems.NOT_LIKE_RIGHT,
								MenuItems.NOT_LIKE_LEFT_RIGHT));
					}
				}

				if (!getDTable().filtering().isFiltered(column)) {
					menuItems.addAll(Arrays.asList(MenuItems.IS_EMPTY,
							MenuItems.IS_NOT_EMPTY));
				}

				for (MenuItems menuItem : menuItems) {
					String filter = getDTable().translation().get(
							menuItem.description());
					filter = (filter != null ? filter : menuItem.description())
							.replace("_", cellValue);

					if (menuItem.description().equals(
							MenuItems.IS_EMPTY.description())) {
						if (!getDTable().filtering().filteringExists(column,
								MenuItems.IS_NOT_EMPTY.description())) {
							item = DOM.createDiv();
							item.setId(menuItem.id());
							item.setInnerText(filter);
							menu.appendChild(item);
						}
					} else if (!menuItem.description().equals(
							MenuItems.LIKE_LEFT_RIGHT.description())
							&& !menuItem.description()
									.equals(MenuItems.NOT_LIKE_LEFT_RIGHT
											.description())) {
						if (!getDTable().filtering().filteringExists(column,
								filter)) {
							item = DOM.createDiv();
							item.setId(menuItem.id());
							item.setInnerText(filter);
							menu.appendChild(item);
						}
					} else if (menuItem.description().equals(
							MenuItems.LIKE_LEFT_RIGHT.description())) {
						for (int i = 1; i <= 5 && i <= cellValue.length(); i++) {
							filter = menuItem.description().replace("_",
									cellValue.substring(0, i));
							if (!getDTable().filtering().filteringExists(
									column, filter)) {
								item = DOM.createDiv();
								item.setId(menuItem.id() + i);
								item.setInnerText(filter);
								menu.appendChild(item);
							}
						}
					} else if (menuItem.description().equals(
							MenuItems.NOT_LIKE_LEFT_RIGHT.description())) {
						for (int i = 1; i <= 5 && i <= cellValue.length(); i++) {
							filter = menuItem.description().replace("_",
									cellValue.substring(0, i));
							if (!getDTable().filtering().filteringExists(
									column, filter)) {
								item = DOM.createDiv();
								item.setId(menuItem.id() + i);
								item.setInnerText(filter);
								menu.appendChild(item);
							}
						}
					}
				}
			}

			if (getDTable().filtering().isFiltered(column)
					|| getDTable().searchColumns().isSearched(column)) {
				String headerId = "table#dt-"
						+ (getDTable().isFrozen(column) ? "frozen-" : "")
						+ "header-table" + id[0] + " tr";
				String header = $(headerId)
						.children("td [id='" + id[0] + "h:0:" + id[2] + "']")
						.children("div").first().text();
				item = DOM.createDiv();
				item.setId(MenuItems.FILTER_REMOVE.id());
				item.setInnerText(getDTable().translation()
						.get(MenuItems.FILTER_REMOVE.description())
						.replace("_", header));
				menu.appendChild(item);
			}
		}
	}

	@Override
	public void onBrowserEvent(Event event) {
		event.stopPropagation();

		switch (DOM.eventGetType(event)) {
		case Event.ONCLICK:
			hide();
			String menuItem = DOM.eventGetTarget(event).getId();
			if (menuItem.startsWith(MenuItems.LIKE_LEFT_RIGHT.id())) {
				menuItem = MenuItems.LIKE_LEFT_RIGHT.id();
			} else if (menuItem.startsWith(MenuItems.NOT_LIKE_LEFT_RIGHT.id())) {
				menuItem = MenuItems.NOT_LIKE_LEFT_RIGHT.id();
			}

			switch (MenuItems.item(menuItem)) {
			case SORTING_ASCENDING:
				getDTable().sorting().setSortings($(targetE), "asc");
				getDTable().sendMessage("sorting",
						getDTable().sorting().getSortings(), true);
				break;

			case SORTING_DESCENDING:
				getDTable().sorting().setSortings($(targetE), "desc");
				getDTable().sendMessage("sorting",
						getDTable().sorting().getSortings(), true);
				break;

			case SORTING_REMOVE:
				getDTable().sorting().setSortings($(targetE), "remove");
				getDTable().sendMessage("sorting",
						getDTable().sorting().getSortings(), true);
				break;

			case SORTING_REMOVE_ALL:
				getDTable().sorting().setSortings($(targetE), null);
				getDTable().sendMessage("sorting",
						getDTable().sorting().getSortings(), true);
				break;

			case EQUAL:
			case NOT_EQUAL:
			case GREATER:
			case GREATER_EQUAL:
			case LESS:
			case LESS_EQUAL:
			case LIKE:
			case NOT_LIKE:
			case LIKE_LEFT:
			case LIKE_RIGHT:
			case LIKE_LEFT_RIGHT:
			case NOT_LIKE_LEFT:
			case NOT_LIKE_RIGHT:
			case NOT_LIKE_LEFT_RIGHT:
			case IS_EMPTY:
			case IS_NOT_EMPTY:
				cellValue = "&nbsp".equals(cellValue) ? "" : cellValue;
				// column, row, operator, filter, cell value
				getDTable().filtering().newFiltering(column, row,
						MenuItems.item(menuItem).operator(),
						DOM.eventGetTarget(event).getInnerText(), cellValue);
				break;

			case FREEZE:
			case UNFREEZE:
				getDTable().setFrozen(column);
				break;

			case HIDE:
				getDTable().sendMessage("hideColumn", "" + column, true);
				break;

			case FILTER_REMOVE:
				getDTable().filtering().removeFiltering(column);
				break;

			case NONE:
			default:
				break;
			}
			break;

		case Event.ONCONTEXTMENU:
			event.preventDefault();
			break;

		default:
			break;
		}
	}

	@Override
	public void show() {
		if (menu.getChildCount() > 0) {
			super.show();

			int popupLeft = (getAbsoluteLeft() + getOffsetWidth() + 5) <= Window
					.getClientWidth() ? getPopupLeft() : (Window
					.getClientWidth() - getOffsetWidth() - 5);
			int popupTop = (getAbsoluteTop() + getOffsetHeight() + 5) <= Window
					.getClientHeight() ? getPopupTop() : (Window
					.getClientHeight() - getOffsetHeight() - 5);

			setPopupPosition(popupLeft, popupTop);
		}
	}

}
