/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.config;

import org.dussan.vaadin.dtable.config.tablesection.TableSectionConfig;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlign;
import org.dussan.vaadin.dtable.config.tablesection.alignments.RowAlign;

public class HeaderConfig extends TableSectionConfig {

	private boolean enableAllColumns = false;
	private boolean enableExpandedFrozenCellForActiveColumn = false;
	private boolean enableExpandedCellForActiveColumn = false;

	private boolean enableTooltip = false;
	private boolean enableWordWrapInTooltip = false;
	private TooltipPosition tooltipPosition = null;
	private int tooltipMinimumWidth = 0;
	private int tooltipMaximumWidth = 0;

	public HeaderConfig() {
		setDefaultColumnAlignment(ColumnAlign.CENTER);
		setRowAlignment(RowAlign.MIDDLE);
		setTooltipPosition(TooltipPosition.BOTTOM);
	}

	public boolean isEnabledAllColumns() {
		return enableAllColumns;
	}

	public void enableAllColumns(boolean enableShowingAllColumnsHeader) {
		this.enableAllColumns = enableShowingAllColumnsHeader;
	}

	public boolean isEnabledExpandedFrozenCellForActiveColumn() {
		return enableExpandedFrozenCellForActiveColumn;
	}

	public void enableExpandedFrozenCellForActiveColumn(
			boolean enableExpandedFrozenCellForActiveColumn) {
		this.enableExpandedFrozenCellForActiveColumn = enableExpandedFrozenCellForActiveColumn;
	}

	public boolean isEnabledExpandedCellForActiveColumn() {
		return enableExpandedCellForActiveColumn;
	}

	public void enableExpandedCellForActiveColumn(
			boolean enableExpandedCellForActiveColumn) {
		this.enableExpandedCellForActiveColumn = enableExpandedCellForActiveColumn;
	}

	public boolean isEnabledTooltip() {
		return enableTooltip;
	}

	public void enableTooltip(boolean enableTooltip) {
		this.enableTooltip = enableTooltip;
	}

	public TooltipPosition getTooltipPosition() {
		return tooltipPosition;
	}

	public void setTooltipPosition(TooltipPosition tooltipPosition) {
		this.tooltipPosition = tooltipPosition;
	}

	public boolean isEnabledWordWrapInTooltip() {
		return enableWordWrapInTooltip;
	}

	public void enableWordWrapInTooltip(boolean enableWordWrapInTooltip) {
		this.enableWordWrapInTooltip = enableWordWrapInTooltip;
	}

	public void setTooltipWidth(int tooltipMinimumWidth, int tooltipMaximumWidth) {
		setTooltipMinimumWidth(tooltipMinimumWidth);
		setTooltipMaximumWidth(tooltipMaximumWidth);
	}

	public int getTooltipMinimumWidth() {
		return tooltipMinimumWidth;
	}

	public void setTooltipMinimumWidth(int tooltipMinimumWidth) {
		this.tooltipMinimumWidth = tooltipMinimumWidth;
	}

	public int getTooltipMaximumWidth() {
		return tooltipMaximumWidth;
	}

	public void setTooltipMaximumWidth(int tooltipMaximumWidth) {
		this.tooltipMaximumWidth = tooltipMaximumWidth;
	}

	@Override
	public boolean isEnabledWordWrap() {
		// TODO enable for header
		return Boolean.FALSE;
	}

}
