/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.translations;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

public class Translation implements Serializable {

	private static final long serialVersionUID = -6496946751885167174L;
	private Map<Locale, Map<String, String>> translatedWords = new HashMap<Locale, Map<String, String>>();

	public Translation() {
		Map<String, String> translatedWords = new HashMap<String, String>();
		translatedWords.put(TranslatedWord.ACTION.toString(), "ACTION");
		translatedWords.put(
				TranslatedWord.ASCENDING_SORTING_FOR_COLUMN.toString(),
				"ASCENDING SORTING FOR COLUMN");
		translatedWords
				.put(TranslatedWord.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_UNHIDDEN
						.toString(), "At least one column have to be unhidden!");
		translatedWords.put(
				TranslatedWord.AT_LEAST_ONE_FROZEN_COLUMN_HAVE_TO_BE_UNHIDDEN
						.toString(),
				"At least one frozen column have to be unhidden!");
		translatedWords
				.put(TranslatedWord.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_FROZEN_AND_UNHIDDEN
						.toString(),
						"At least one column have to be frozen and unhidden!");
		translatedWords
				.put(TranslatedWord.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_UNFROZEN_AND_UNHIDDEN
						.toString(),
						"At least one column have to be unfrozen and unhidden!");
		translatedWords.put(TranslatedWord.CLOSE.toString(), "CLOSE");
		translatedWords.put(TranslatedWord.CSV_FILE.toString(), "CSV file");
		translatedWords.put(
				TranslatedWord.DATA_ARE_READY_FOR_DOWNLOAD.toString(),
				"Data are ready for download!");
		translatedWords.put(TranslatedWord.DATA_ERROR.toString(), "DATA ERROR");
		translatedWords.put(
				TranslatedWord.DESCENDING_SORTING_FOR_COLUMN.toString(),
				"DESCENDING SORTING FOR COLUMN");
		translatedWords.put(TranslatedWord.DOWNLOAD_THE_DATA.toString(),
				"DOWNLOAD THE DATA");
		translatedWords.put(TranslatedWord.ERROR.toString(), "ERROR");
		translatedWords.put(TranslatedWord.EXPORT.toString(), "EXPORT");
		translatedWords
				.put(TranslatedWord.EXPORTED_COLUMN_DATA_TYPE_AND_COLUMN_DEFINITION_TYPE_ARE_NOT_THE_SAME
						.toString(),
						"Exported column data type and column definition type are not the same!");
		translatedWords.put(
				TranslatedWord.EXPORTING_DATA_TO_CSV_FILE.toString(),
				"Exporting data (CSV file)");
		translatedWords.put(
				TranslatedWord.EXPORTING_DATA_TO_MS_ACCESS_FILE.toString(),
				"Exporting data (MS Access file)");
		translatedWords.put(
				TranslatedWord.EXPORTING_DATA_TO_MS_EXCEL_FILE.toString(),
				"Exporting data (MS Excel file)");
		translatedWords.put(
				TranslatedWord.EXPORTING_DATA_TO_PDF_FILE.toString(),
				"Exporting data (PDF file)");
		translatedWords.put(TranslatedWord.FILTERS.toString(), "FILTERS:");
		translatedWords.put(TranslatedWord.FREEZE_COLUMN.toString(),
				"FREEZE COLUMN");
		translatedWords.put(TranslatedWord.FREEZE_UNFREEZE_COLUMNS.toString(),
				"Freeze/unfreeze columns");
		translatedWords.put(TranslatedWord.FROZEN.toString(), "frozen");
		translatedWords.put(TranslatedWord.FROZEN_COLUMNS.toString(),
				"Frozen columns");
		translatedWords.put(TranslatedWord.HIDE_COLUMN.toString(),
				"HIDE COLUMN");
		translatedWords.put(TranslatedWord.HIDDEN.toString(), "hidden");
		translatedWords.put(TranslatedWord.HIDDEN_COLUMNS.toString(),
				"Hidden columns");
		translatedWords.put(TranslatedWord.HIDE_UNHIDE_COLUMNS.toString(),
				"Hide/unhide columns");
		translatedWords.put(
				TranslatedWord.IN_PREPARING_DATA_AN_ERROR_OCCURRED.toString(),
				"In preparing data an error occurred!");
		translatedWords.put(TranslatedWord.IS_EMPTY.toString(), "IS EMPTY");
		translatedWords.put(TranslatedWord.IS_NOT_EMPTY.toString(),
				"IS NOT EMPTY");
		translatedWords.put(TranslatedWord.LOADING_DATA_PLEASE_WAIT.toString(),
				"Loading data, please wait...");
		translatedWords.put(TranslatedWord.MS_ACCESS_FILE.toString(),
				"MS Access file");
		translatedWords.put(TranslatedWord.MS_EXCEL_FILE.toString(),
				"MS Excel file");
		translatedWords.put(TranslatedWord.NO_DATA_TO_DISPLAY.toString(),
				"No data to display!");
		translatedWords.put(TranslatedWord.NO_DATA_FOR_EXPORT.toString(),
				"No data for export!");
		translatedWords.put(TranslatedWord.NO_NEW_DATA.toString(),
				"NO NEW DATA");
		translatedWords.put(TranslatedWord.OK.toString(), "OK");
		translatedWords.put(TranslatedWord.PDF_FILE.toString(), "PDF file");
		translatedWords.put(TranslatedWord.PREPARING_DATA.toString(),
				"Preparing data");
		translatedWords.put(
				TranslatedWord.PREPARING_DATA_FOR_DOWNLOAD.toString(),
				"Preparing data for download...");
		translatedWords.put(
				TranslatedWord.PREPARING_DATA_IS_COMPLETED.toString(),
				"Preparing data is completed!");
		translatedWords.put(
				TranslatedWord.PREPARING_DATA_IS_STOPPED.toString(),
				"Preparing data is stopped!");
		translatedWords.put(TranslatedWord.QUICK_SEARCH.toString(),
				"Quick search");
		translatedWords.put(TranslatedWord.REMOVE_ALL_SORTINGS.toString(),
				"REMOVE ALL SORTINGS");
		translatedWords.put(
				TranslatedWord.REMOVE_FILTERS_FROM_COLUMN.toString(),
				"REMOVE FILTERS FROM COLUMN");
		translatedWords.put(
				TranslatedWord.REMOVE_SORTING_FOR_COLUMN.toString(),
				"REMOVE SORTING FOR COLUMN");
		translatedWords.put(TranslatedWord.SEARCH.toString(), "SEARCH:");
		translatedWords.put(TranslatedWord.SEARCH_IN_COLUMNS.toString(),
				"Search in columns");
		translatedWords.put(
				TranslatedWord.SIZE_OF_THE_REQUESTED_DATA_IS_ZERO.toString(),
				"Size of the requested data is zero!");
		translatedWords
				.put(TranslatedWord.SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS
						.toString(),
						"Something is wrong in data preparation, please check your data settings!");
		translatedWords.put(TranslatedWord.STOP_PREPARING_DATA.toString(),
				"STOP PREPARING DATA");
		translatedWords.put(
				TranslatedWord.TO_EXPORT_THE_DATA_PRESS_THE_BUTTON_EXPORT
						.toString(),
				"To export the data press the button \"EXPORT\"!");
		translatedWords.put(TranslatedWord.UNFREEZE_COLUMN.toString(),
				"UNFREEZE COLUMN");
		translatedWords.put(TranslatedWord.UNFROZEN_COLUMNS.toString(),
				"Unfrozen columns");
		translatedWords.put(TranslatedWord.UNHIDDEN_COLUMNS.toString(),
				"Unhidden columns");
		translatedWords.put(
				TranslatedWord.WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED
						.toString(),
				"When transmitting data an error has occurred!");
		translatedWords.put(
				TranslatedWord.WINDOW_FOR_EXPORTING_DATA_IS_ALREADY_OPEN
						.toString(),
				"Window for exproting data is already open!");
		translatedWords
				.put(TranslatedWord.WINDOW_FOR_HIDING_UNHIDING_COLUMNS_DATA_IS_ALREADY_OPEN
						.toString(),
						"Window for hiding/unhiding columns is already open!");

		// save translations
		this.translatedWords.put(new Locale("en"), translatedWords);
	}

	public String get(Locale locale, TranslatedWord translatedWord) {
		try {
			if (translatedWords.containsKey(locale)
					&& translatedWords.get(locale).containsKey(
							translatedWord.toString())) {
				return translatedWords.get(locale).get(
						translatedWord.toString());
			}
			return translatedWords.get(new Locale("en")).get(
					translatedWord.toString());
		} catch (Exception e) {
		}
		return "_TRANSLATION_ERROR_";
	}

	public void set(Locale locale, TranslatedWord translateWord,
			String translatedWord) {
		if (locale != null && translatedWord != null
				&& !translatedWord.isEmpty()) {
			Map<String, String> translatedWords = get(locale);
			if (translatedWords == null) {
				translatedWords = new HashMap<String, String>();
			}
			translatedWords.put(translateWord.toString(), translatedWord);
			this.translatedWords.put(locale, translatedWords);
		}
	}

	public Map<String, String> get(Locale locale) {
		if (translatedWords.containsKey(locale)) {
			translatedWords.get(locale);
		}
		return translatedWords.get(new Locale("en"));
	}

	public void set(Locale locale, Map<String, String> translatedWords) {
		if (locale != null && translatedWords != null
				&& !translatedWords.isEmpty()) {
			this.translatedWords.put(locale, translatedWords);
		}
	}

	public Map<String, String> getPaintObjects(Locale locale) {
		if (locale != null && translatedWords.get(locale) != null
				&& !translatedWords.get(locale).isEmpty()) {
			return translatedWords.get(locale);
		}
		return translatedWords.get(new Locale("en"));
	}

}
